###############################################################################################
# $Id: setup_client.py,v 1.1.1.1 2002/02/19 10:16:05 slmm Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import os
import getpass
import traceback
import re
import scenario
import properties
from socket         import *
from net.connection import Connection
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.label          import Label
from gui.editfield      import EditField
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox

# return codes used to indicate what should happen
CANCEL = 0
OK     = 1

class SetupClient(Dialog):
    """
    This class is used as a dialog for letting the player select wether to run as a client or a
    server. A name can also be given by the players. This is a human readable name that is used for
    the communication between the players. 
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )
       

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( Label (self.titlefont, "Setup network information", (10, 10), color=(255, 0, 0)))

        self.wm.register ( Label (self.smallfont, "My name: ",       (250, 240), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Run as server: ", (250, 300), color=(255, 255, 255)) )

        # get the user name from the environment if it works
        try:
            name = os.environ['USER']
        except:
            name = "My name"

        # the editfields
        self.user = EditField ( self.smallfont, name,    200, (450, 230), color=(255,255,0),
                                background = (80,80,80))

        self.server = CheckBox ( "Server", self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=1, position=(450, 290), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )
        self.client = CheckBox ( "Client", self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=0, position=(570, 290), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )

        self.wm.register ( self.user )
        self.wm.register ( self.server )
        self.wm.register ( self.client )

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-quit-moff.png",
                                    properties.path_dialogs + "butt-quit-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.cancel } ) )

    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog. """
        
        # store our name, if we got anything
        if self.user.getText () == "":
            # nothing there, use the login name
            scenario.local_player_name = getpass.getuser ()
        else:
            scenario.local_player_name = self.user.getText ()
        
        # we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE

   
                
    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def toggleServer (self, trigger, event):
        """Callback triggered when one of the client/server checkboxes are clicked. This method
        makes sure the  other checkboxes are unchecked."""
        # who triggered the event?
        if trigger == self.server:
            # disable the other button
            self.client.setChecked ( 0 )

            # running as server
            scenario.run_as_server = 1
        else:
            # disable the other button
            self.server.setChecked ( 0 )

            # running as client
            scenario.run_as_server = 0

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: end_game.py,v 1.1.1.1 2002/02/19 10:16:08 slmm Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import os
import scenario
import constants
import properties
import gui.widget
from gui.fader          import Fader
from gui.dialog         import *
from gui.button         import Button
from gui.label          import Label
from gui.widget_manager import WidgetManager
from organization       import UNION, REBEL, UNKNOWN


class EndGame (Dialog):
    """
    This class is used as a dialog for showing the final results of the game to the player. This
    class will gather statistics about the game, such as casualities and captured
    objectives. Depending on how the game ended a different backdrop will be used, i.e. the winning
    player gets a different backdrop from the losing player.

    First only the backdrop is shown, no stats. Then after a while a part starts to fade towards
    black, and then the stats are shown on that one.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # calculate statistics
        self.calculateStatistics ()

        # init superclass
        Dialog.__init__ (self, scenario.sdl)
        
        # set our background to a tiled image. this should take into account wether the player won
        # or lost and use a different background for a loss and victory
        self.setBackground ( properties.endgame_background )

        # we're showing backdrop. this is a counter (seconds) that only the backdrop is shown
        self.showbackdroponly = 4
        
        # we want time events
        self.enableTimer ( 1000 )
        

    def createWidgets (self):
        "Creates all widgets for the dialog."
 
        # load the fade and the box
        self.fader = Fader ( position=(212, 100), size=(600, 300), alpha=0 )
        
        # register our fader
        self.wm.register ( self.fader )


    def createLabels (self):
        "Creates all labels for the dialog."
        # create the labels
        report = Label (self.titlefont, "Battle report", (400, 120), color=(255, 255, 0))
        result = Label (self.smallfont, self.summary,    (400, 160), color=(255, 255, 255))

        # create the side headers
        men    = Label (self.titlefont, "Men",           (250, 240), color=(255, 180, 100))
        guns   = Label (self.titlefont, "Guns",          (250, 270), color=(255, 180, 100))
        score  = Label (self.titlefont, "Score",         (250, 320), color=(255, 180, 100))

        # create the top headers
        union = Label (self.titlefont, "Union",          (350, 200), color=(255, 255, 0))
        rebel = Label (self.titlefont, "Confederate",    (500, 200), color=(255, 255, 0))
        
        # create result strings for mens and guns
        unionmenstr  = "%d of %d" % ( self.men_destroyed [UNION],  self.men_ok[UNION] )
        rebelmenstr  = "%d of %d" % ( self.men_destroyed [REBEL],  self.men_ok[REBEL] )
        uniongunsstr = "%d of %d" % ( self.guns_destroyed [UNION], self.guns_ok[UNION] )
        rebelgunsstr = "%d of %d" % ( self.guns_destroyed [REBEL], self.guns_ok[REBEL] )
        scorestr     = "%d vs %d" % ( self.score [UNION],          self.score [REBEL] )

        # and labels too
        unionmen  = Label (self.smallfont, unionmenstr,  (350, 250), color=(255, 255, 255))
        unionguns = Label (self.smallfont, uniongunsstr, (350, 280), color=(255, 255, 255))
        rebelmen  = Label (self.smallfont, rebelmenstr,  (500, 250), color=(255, 255, 255))
        rebelguns = Label (self.smallfont, rebelgunsstr, (500, 280), color=(255, 255, 255))
        score2    = Label (self.smallfont, scorestr,     (425, 325), color=(255, 255, 255))
        
        # register our labels
        self.wm.register ( report )
        self.wm.register ( result )
        self.wm.register ( union )
        self.wm.register ( rebel )
        self.wm.register ( men )
        self.wm.register ( guns )
        self.wm.register ( score )
        self.wm.register ( score2 )
        self.wm.register ( unionmen )
        self.wm.register ( rebelmen )
        self.wm.register ( unionguns )
        self.wm.register ( rebelguns )
      
        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (406, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )

      
   
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button."""

        # no more timer events needed
        self.disableTimer ()
       
        # we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE


    def timer (self):
        """Callback triggered when the dialog has enabled timers and a timer fires. Updates the fade
        image on the dialog with some more alpha."""

        # are we still showing the backdrop only?
        if self.showbackdroponly > 0:
            # yes, decrement and go away
            self.showbackdroponly -= 1

            # did we get to 0 now?
            if self.showbackdroponly == 0:
                # yes, so change the timer value to make it faster
                self.enableTimer ( 100 )
 
            return
        
        # we've shown the backdrop for some seconds, now get on with the fading. first get old alpha 
        alpha = self.fader.getAlpha ()

        # are we done fading?
        if alpha >= 50:
            # yep, no more fading needed
            self.disableTimer ()

            # create labels
            self.createLabels ()

        else:
            # fade more, andd 5 to the alpha
            self.fader.setAlpha ( alpha + 5 )
        

    def calculateStatistics (self):
        """This method calculates statistics about how the battle went. The data that is gathered
        is:

        * total/lost men
        * total/lost guns
        * total/lost units
        * captured objectives
        * a victory score

        The stats are stored in members for later usage.
        """
        # calculate destroyed units
        self.calculateDestroyedUnits ()

        # and men
        self.calculateDestroyedMenAndGuns ()

        # calculate all objectives
        self.calculateObjectives ()
        
        # now calculate a victory scrore
        self.calculateScore ()

        # create a summary string
        self.createSummary ()
        

    def calculateDestroyedUnits (self):
        """This method calculates the number of units for both players that have been destroyed and
        how many are still ok, i.e. not destroyed."""
        
        # nothing alive at all yet
        self.units_ok        = { REBEL: 0, UNION: 0 }
        self.units_destroyed = { REBEL: 0, UNION: 0 }
        
        # loop over all ok units
        for unit in scenario.units.values():
            # add to the counts
            self.units_ok [unit.getOwner ()] += 1

        # loop over all destroyed units
        for unit in scenario.destroyed_units.values():
            # add to the counts
            self.units_destroyed [unit.getOwner ()] += 1


    def calculateDestroyedMenAndGuns (self):
        """This method calculates the number of men and guns for both players that have been
        destroyed and how many are still ok, i.e. not destroyed."""
        
        # nothing alive at all yet
        self.men_ok         = { REBEL: 0, UNION: 0 }
        self.men_destroyed  = { REBEL: 0, UNION: 0 }
        self.guns_ok        = { REBEL: 0, UNION: 0 }
        self.guns_destroyed = { REBEL: 0, UNION: 0 }
        
        # loop over all ok units
        for unit in scenario.units.values():
            # add to the counts
            self.men_ok        [unit.getOwner ()] += unit.getMen ()
            self.men_destroyed [unit.getOwner ()] += unit.getKilled ()

            # does it have guns?
            if unit.hasGuns ():
                # yep, add those too
                self.guns_ok        [unit.getOwner ()] += unit.getGuns ()
                self.guns_destroyed [unit.getOwner ()] += unit.getDestroyedGuns ()
                

        # loop over all destroyed units
        for unit in scenario.destroyed_units.values():
            # add to the counts. these units have no ok guns nor men
            self.men_destroyed [unit.getOwner ()] += unit.getKilled ()

            # does it have guns?
            if unit.hasGuns ():
                # yep, add those too
                self.guns_destroyed [unit.getOwner ()] += unit.getDestroyedGuns ()
  

    def calculateObjectives (self):
         """Calculates all data related to objectives. Each objective a player has captured
         increases the score."""

         # no points yet
         self.objectives = { REBEL: 0, UNION: 0 } 

         # loop over all objectives
         for objective in scenario.objectives:
             # is the objective neutral?
             if objective.getOwner () != UNKNOWN:
                 # nope, so add the score for the objective to the proper player
                 self.objectives [objective.getOwner ()] += objective.getPoints ()

                 
    def calculateScore (self):
         """Calculates a final score based on all totals."""

         # no score yet
         self.score = { REBEL: 0, UNION: 0 } 

         # add for all ok and destroyed men
         for player in [ REBEL, UNION ]:
             # add 1 for each killed enemy
             self.score [ player ] += self.men_destroyed [ [UNION, REBEL][player] ] * 1

             print "EndGame.calculateScore: %d: killed: %d" % (player,
                                                               self.men_destroyed [ [UNION, REBEL][player]])
             
             # add 100 for each destroyed enemy gun
             self.score [ player ] += self.guns_destroyed [ [UNION, REBEL][player] ] * 100

             print "EndGame.calculateScore: %d: guns: %d" % (player,
                                                             self.guns_destroyed [ [UNION, REBEL][player]])
 
             # add the scores for the objectives too
             self.score [ player ] += self.objectives [ player ]
              
             print "EndGame.calculateScore: %d: objectives: %d" % (player, self.objectives [player])

         print "EndGame.calculateScore: total: %d - %d" % ( self.score [0], self.score [1] )

         # get a nice sum of the scores
         sum = self.score [0] + self.score [1]

         # get the percentages for the players
         self.score [0] = int ( ( float(self.score[0]) / float(sum) )  * 100 )
         self.score [1] = int ( ( float(self.score[1]) / float(sum) )  * 100 )

         print "EndGame.calculateScore: percentage: %d - %d" % ( self.score [0], self.score [1] )


    def createSummary (self):
        """This method creates a summary string that depends on the way the game ended. In most
        cases the calculated score in calculateScore() is used."""

        self.summary = ""
        
        # how did we end?
        type = scenario.end_game_type

        # special ending texts
        special = { constants.UNION_SURRENDER : 'The Union army surrenders',
                    constants.REBEL_SURRENDER : 'The Confederate army surrenders',
                    constants.UNION_DESTROYED : 'The Union army is destroyed',
                    constants.REBEL_DESTROYED : 'The Confederate army is destroyed',
                    constants.TIMEOUT         : 'Time ran out - ',
                    constants.CEASE_FIRE      : 'Cease fire called - ' }

        # is the ending one that has a special text?
        if special.has_key ( type ):
            # yep, use it as a default text
            self.summary = special [type ]

        # now, should we leave it at that?
        if type in [ constants.UNION_SURRENDER, constants.REBEL_SURRENDER,
                     constants.UNION_DESTROYED, constants.REBEL_DESTROYED ]:
            # yes, these types need no more explanation
            return

        # see the victory level
        rebelscore = self.score[REBEL]
        if rebelscore in range ( 75, 100 ):
            # rebel major victory
            self.summary += "Confederate major victory"

        elif rebelscore in range ( 60, 74 ):
            # rebel major victory
            self.summary += "Confederate minor victory"

        elif rebelscore in range ( 41, 59 ):
            # rebel major victory
            self.summary += "Draw"

        elif rebelscore in range ( 26, 40 ):
            # union minor victory
            self.summary += "Union minor victory"

        else:
            # union major victory
            self.summary += "Union major victory"

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



import os
import pygame, pygame.font, pygame.image, pygame.mixer, pygame.transform
from pygame.locals import *
from gui.dialog import *

pygame.init()

mode = DOUBLEBUF

mode = FULLSCREEN
	
screen = pygame.display.set_mode( (1024,768) )
	
pygame.display.set_caption('4st Attack')
pygame.mouse.set_visible(1)
pygame.display.init()
	
pygame.display.Info()

def quit():
	pygame.quit()
	os._exit(0)

	


class TestDialog(Dialog):
	def __init__ (self, surface):
		Dialog.__init__ (self, surface)
	
	def createWidgets (self):
		self.wm.register ( Label (pygame.font.Font("lucida.ttf", 20), "Setup network information", (10, 10), color=(255, 0, 0)))

def main():
	testd = TestDialog(screen)
	testd.run()

if __name__ == '__main__':
	main()



###############################################################################################
# $Id: checkbox.py,v 1.2 2002/05/21 20:49:51 slmm Exp $
###############################################################################################

from widget import Widget
import widget
import pygame
from pygame.locals import *

# needed for resetting the background
import copy

class CheckBox(Widget):
    """
    This class defines a checkbox which can be in a checked or unchecked state. The two needed
    images will be loaded by the constructor from the passed filenames.

    This class plays a small sound when the checkbox is toggled.
    """

    def __init__ (self, image_checked, image_unchecked, position = (0,0), checked = 0,
                  callbacks = None):
        """Initializes the widget. Loads the icons from the passed filenamea."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        # load the icons
        self.checked   = image_checked 
        self.unchecked = image_unchecked 
        
        # store the default state
        self.state = checked

        # set the surface too, so that isInside() has something to check
        self.surface = self.checked

        # set our internal callbacks so that we can trap changes
        self.internal = {widget.MOUSEBUTTONUP : self.toggle }

	# set the original background to none, is used in the paint method
	self.original_background = None
 

    def isChecked (self):
        """Returns 1 if the checkbox is checked and 0 if it is not checked."""
        return self.state


    def setChecked (self, checked = 1):
        """Sets the checkbox as unchecked if the parameter is 0, and to checked for all other
        values. """
        if checked == 0:
            self.state = 0
        else:
            self.state = 1

        # we're dirty now
        self.dirty = 1


    def toggle (self, event):
        """Toggles the state of a checkbox."""
        if self.state == 0:
            self.state = 1
        else:
            self.state = 0

        # play a sound
        #audio.playSample ( self.sound_toggled )

        # we're dirty now
        self.dirty = 1
        

    def paint (self, destination, force=0):
        """Method that paints the editfield. This method will simply blit out the surface of the widget
        onto the destination surface. """

        if self.original_background is None:
		self.original_background = pygame.Surface(self.checked.get_size())
	        self.original_background.blit(destination.subsurface(
				self.position,
				(self.position[0]+self.checked.get_width(), self.position[1]+self.checked.get_height())
			), (0,0)
		)
	

	
        # are we dirty or not?
        if not self.dirty and not force:
            # not dirty, nothing to do here
            return 0

	destination.blit ( self.original_background, (self.position [0], self.position [1] ) )
        
	# what surface should we use?
        if self.state:
            usedsurface = self.checked
        else:
            usedsurface = self.unchecked
        
        # we're dirty, blit out the button
        destination.blit ( usedsurface, (self.position [0], self.position [1] ) )
             
        self.dirty = 0
        
        # we did something, make sure the widget manager knows that
        return 1

   
    
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



import gui
import pygame
from quit import *
from game import *
from gui.dialog import *
from gui.image import *
from gui.checkbox import *

class SettingsScreen(Dialog):
        def __init__(self, surface, images, locations, ini_settings):
                self.images = images
                self.locations = locations
		self.player = player
		self.ini_settings = ini_settings
		Dialog.__init__(self, surface)
		
	def createWidgets(self):
		self.surface.blit(self.images['startscreen']['background'],(0,0))
	
		self.wm.register(Image(self.images['settingsscreen']['fullscreen'],
			(int(self.locations['settingsscreen']['fullscreen_x']),
			int(self.locations['settingsscreen']['fullscreen_y']))
		))
		
		self.wm.register(CheckBox(self.images['gui']['checkbox_checked'], self.images['gui']['checkbox_unchecked'],
                        (int(self.locations['settingsscreen']['fullscreen_toggle_x']),
                        int(self.locations['settingsscreen']['fullscreen_toggle_y'])),
			checked = self.ini_settings.settings['video']['fullscreen'] == 'yes',
			callbacks={widget.MOUSEBUTTONUP : self.fullscreen }
                ))

		# If sound system is available
		if pygame.mixer.get_init():
			self.wm.register(Image(self.images['settingsscreen']['music'],
				(int(self.locations['settingsscreen']['music_x']),
				int(self.locations['settingsscreen']['music_y']))
			))
			
			self.wm.register(CheckBox(self.images['gui']['checkbox_checked'], self.images['gui']['checkbox_unchecked'],
				(int(self.locations['settingsscreen']['music_toggle_x']),
				int(self.locations['settingsscreen']['music_toggle_y'])),
				checked = self.ini_settings.settings['sound']['music'] == 'yes',
				callbacks={widget.MOUSEBUTTONUP : self.music }
			))

		self.wm.register(Button(self.images['settingsscreen']['return'], self.images['settingsscreen']['return'],
			(int(self.locations['settingsscreen']['return_x']),
			int(self.locations['settingsscreen']['return_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.return_to_main }
                ))

        def return_to_main(self, trigger, event):
		self.state = 1
		return widget.DONE
		
	def fullscreen(self, trigger, event):
		if not (self.ini_settings.settings['video']['fullscreen'] == 'yes'):
			self.ini_settings.settings['video']['fullscreen'] = 'yes'
			pygame.display.toggle_fullscreen()
		else:
			self.ini_settings.settings['video']['fullscreen'] = 'no'
			pygame.display.toggle_fullscreen()
		self.ini_settings.save()
		
		self.wm.paint(1,0)
		
	def music(self, trigger, event):
		
		if not (self.ini_settings.settings['sound']['music'] == 'yes'):
			self.ini_settings.settings['sound']['music'] = 'yes'
			pygame.mixer.music.load( os.path.join(self.ini_settings.settings['path']['data'], 'music', '4stattack.ogg') )
			pygame.mixer.music.play(-1)
		else:
			self.ini_settings.settings['sound']['music'] = 'no'
			pygame.mixer.music.stop()
			
		self.ini_settings.save()
		
		self.wm.paint(1,0)



###############################################################################################
# $Id: setup_network.py,v 1.1.1.1 2002/02/19 10:16:02 slmm Exp $
###############################################################################################

import pygame
import pygame.display
import pygame.surface
import os
import getpass
import traceback
import re
import scenario
import scenario_parser
import properties
from socket         import *
from net.connection import Connection
import gui.widget
from gui.image          import Image
from gui.dialog         import *
from gui.button         import Button
from gui.checkbox       import CheckBox
from gui.label          import Label
from gui.editfield      import EditField
from gui.widget_manager import WidgetManager
from gui.messagebox     import Messagebox
from gui.wait_client    import WaitClient

# return codes used to indicate what should happen
CANCEL = 0
OK     = 1

class SetupNetwork(Dialog):
    """
    This class is used as a dialog for setting up the connection to the game server. It asks for the
    user name (whatever name) and the name and port of the host where the server runs. The network
    connection is then initialized if all is ok.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )
       

    def createWidgets (self):
        """Creates all widgets for the dialog. Depending on wether we're a server or a client
        different widgets will be shown."""

        # are we a server?
        if not scenario.run_as_server:
            # we're running as client. need an additional label and editfield for the hostname
            self.wm.register ( Label (self.smallfont, "Server host: ",   (250, 240), color=(255, 255, 255)) )

            self.host = EditField ( self.smallfont, "localhost", 200, (450, 240), color=(255,255,0),
                                    background = (80,80,80))

            self.wm.register ( self.host )

        # common widgets
        self.wm.register ( Label (self.titlefont, "Setup network information", (10, 10), color=(255, 0, 0)))

        self.wm.register ( Label (self.smallfont, "Server port: ",   (250, 300), color=(255, 255, 255)) )
        self.port = EditField ( self.smallfont, str(properties.network_port), 200, (450, 300),
                                color=(255,255,0), background = (80,80,80))

        self.wm.register ( self.port )
           
        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-back-moff.png",
                                    properties.path_dialogs + "butt-back-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.back } ) )

    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog. Tries to connect to the remote server or
        waits for a remote client to connect to us, and if all is ok the dialog is accepted. If
        something fails the player may choose again. If some data is missing the player must choose
        again"""

        # get the port. this is common for both client and server
        try:
            port = int ( self.port.getText () )

            # is it valid?
            if not 1 <= port <= 65535:
                # not a valid number, show a messagebox
                Messagebox ( "Invalid port number, must be in the range 1 to 65535!" )

                # repaint and go away
                self.wm.paint (force=1, clear=1)
                return widget.HANDLED
            
        except TypeError:
            # not a valid number, show a messagebox
            Messagebox ( "Invalid port number!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED


        # are we a server?
        if scenario.run_as_server:
            # init serverside socket
            return self.initServerSocket ( port )

        else:
            # init clientside socket
            return self.initClientSocket ( port )


    def initServerSocket (self, port):
        """ """
        try:
            # create the socket
            s = socket (AF_INET, SOCK_STREAM)
            
            # allow bind()ing while a previous socket is still in TIME_WAIT.
            s.setsockopt ( SOL_SOCKET, SO_REUSEADDR, 1 )
            
            # bind our socket for incoming requests
            s.bind ( ( 'localhost', port ) )
        except:
            # not a valid number, show a messagebox
            Messagebox ( "Could not use socket! It may already be in use." )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED

        # listen for clients
        s.listen ( 1 )

        # run a dialog that shows some nice info while we wait for the client to connect
        dialog = WaitClient ( s )
        dialog.run ()

        # set our status to the status of the executed dialog
        self.state = dialog.state

        # we're done anyway
        return widget.DONE


    def initClientSocket (self, port):
        """Connects to the server. If all is ok a connection is stored in the scenario. On error a
        dialog is shown to the user and the user may try again."""
        
        # get the name of the host
        host = self.host.getText ()

        # did we get a hostname?
        if host == "":
            # oops, no host given, show a messagebox
            Messagebox ( "No server hostname given!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
            return widget.HANDLED

        try:
            # create the socket
            newSocket = socket (AF_INET, SOCK_STREAM)
        
            # connect to the remote system
            newSocket.connect ( ( host, port  ) )

            # all ok, store the new and connected socket and the extra info
            scenario.connection = Connection ( socket=newSocket, host=host, port=port )

        except:
            Messagebox ( "Could not connect to server!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)

            return widget.HANDLED
       
        # all is ok, we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE

               
    def back (self, trigger, event):
        """Callback triggered when the user clicks the 'Back' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes. This is used when the player wants to
        change something."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



###############################################################################################
# $Id: setup_players.py,v 1.1.1.1 2002/02/19 10:16:06 slmm Exp $
###############################################################################################

import os
import getpass
import re
import organization
import scenario
import scenario_parser
import properties
import widget
from dialog import *
from button import Button
from checkbox import CheckBox
from label import Label
from editfield import EditField
from widget_manager import WidgetManager

# return codes used to indicate what should happen
CANCEL = 0
OK     = 1

class SetupPlayers(Dialog):
    """
    This class is used as a dialog for setting up the players for the game. This information
    includes the side the local player will play as (rebel/union), the network information of the
    remote server (hostname) as well as which machine acts as a server. By default the rebel machine
    is the server.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )
       

    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( Label (self.titlefont, "Setup player information", (10, 10), color=(255, 0, 0)))

        self.wm.register ( Label (self.smallfont, "My name: ",       (50, 190), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Remote host: ",   (50, 250), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Play as: ",       (50, 300), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Act as server: ", (50, 340), color=(255, 255, 255)))


        # the editfields
        self.username = EditField ( self.smallfont, "chakie", 200, (200, 180), color=(255,255,0),
                                    background = (80,80,80))
        self.hostname = EditField ( self.smallfont, "localhost", 200, (200, 240), color=(255,255,0),
                                    background = (80,80,80))
        self.wm.register ( self.username )
        self.wm.register ( self.hostname )

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                    properties.path_dialogs + "butt-ok-mover.png",
                                    (284, 650 ), {widget.MOUSEBUTTONUP : self.ok } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (528, 650 ), {widget.MOUSEBUTTONUP : self.cancel } ) )

        # checkboxes
        self.union = CheckBox ( "Union", self.smallfont,
                                properties.path_dialogs + "butt-radio-set.png",
                                properties.path_dialogs + "butt-radio-unset.png",
                                checked=1, position=(200, 300), color=(255,255,0),
                                callbacks={widget.MOUSEBUTTONUP : self.togglePlayer } )
        self.rebel = CheckBox ( "Rebel", self.smallfont,
                                properties.path_dialogs + "butt-radio-set.png",
                                properties.path_dialogs + "butt-radio-unset.png",
                                checked=0, position=(300, 300), color=(255,255,0),
                                callbacks={widget.MOUSEBUTTONUP : self.togglePlayer } )
        self.server = CheckBox ( "Server", self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=1, position=(200, 340), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )
        self.client = CheckBox ( "Client", self.smallfont,
                                 properties.path_dialogs + "butt-radio-set.png",
                                 properties.path_dialogs + "butt-radio-unset.png",
                                 checked=0, position=(300, 340), color=(255,255,0),
                                 callbacks={widget.MOUSEBUTTONUP : self.toggleServer } )

        self.wm.register ( self.rebel ) 
        self.wm.register ( self.union )
        self.wm.register ( self.server ) 
        self.wm.register ( self.client )



    def togglePlayer (self, trigger, event):
        """Callback triggered when one of the checkboxes are clicked. This method makes sure the
        other checkboxes are unchecked."""
        # who triggered the event?
        if trigger == self.union:
            # disable the other button
            self.rebel.setChecked ( 0 )

        else:
            # disable the other button
            self.union.setChecked ( 0 )

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)
            

    def toggleServer (self, trigger, event):
        """Callback triggered when one of the client/server checkboxes are clicked. This method
        makes sure the  other checkboxes are unchecked."""
        # who triggered the event?
        if trigger == self.server:
            # disable the other button
            self.client.setChecked ( 0 )

        else:
            # disable the other button
            self.server.setChecked ( 0 )

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)
            
    
    def ok (self, trigger, event):
        """Callback triggered when the user clicks the 'Ok' button. Applies the changes after
        verifying the given data, and closes the dialog."""
        # store the passed data, first the side we have
        if self.rebel.isChecked ():
           # we're playing as rebel 
           scenario.local_player_id = organization.REBEL
        else:
           # we're playing as union 
           scenario.local_player_id = organization.UNION

        # are we a server?
        if self.server.isChecked ():
           # we're the server
           scenario.act_as_server = 1
        else:
           # we're the client
           scenario.act_as_server = 0

        # store our name, if we got anything
        if self.username.getText () == "":
            # nothing there, use the login name
            scenario.local_player_name = getpass.getuser ()
        else:
            scenario.local_player_name = self.username.getText ()

        # did we get a hostname?
        if self.hostname.getText () == "":
            # oops, no host given
            print "No host given"
            return widget.HANDLED

        # store the name of the rmeote host
        scenario.remote_host = self.hostname.getText ()
        
        # is the host valid?
#        if self.checkHost ( self.hostname.getText () ):
#            # all is ok
#            scenario.remote_host = self.hostname.getText ()
#        else:
#            print "Could not reach remote host"
#            return widget.HANDLED
#        
        # we're accepting the dialog
        self.state = ACCEPTED
        
        return widget.DONE


    def checkHost (self, hostname):
        """Verifies that the given host can be reached by running ping against it. This is really
        crude and should be changed later. But we can't connect yet, so we just verify that the host
        exists at all. This does of course not guarantee that the host is the correct one. Returns 1
        on success and 0 on failure."""

        # define the regexp
        pingRegex = r'.*(?P<trans>\d+)\s+packets transmitted.*\s+' + \
                    r'(?P<recv>\d+)\s+packets received.*\s+' + \
                    r'(?P<loss>\d+)% packet loss'

        # run the external ping
        ping = os.popen ('ping -c 2 -i 1 ' + hostname + ' 2>&1')
        data = ping.read ()

        # does what we got match the expression
        match = re.match (pingRegex, data, re.DOTALL)
        if match:
            # do we have any lost packets?
            if match.group('loss') != '0':
                print "Ping failure: " 
                return 0

        else:
            # did not match regexp
            print "Ping error "
            return 0

        # all is ok as we got this far
        return 1
    
                
    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



# Echo client program
import socket

HOST = ''    # The remote host
PORT = 50007              # The same port as used by the server
s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.connect((HOST, PORT))
s.send('Hello, world')
data = s.recv(1024)
s.close()
print 'Received', `data`





import rules
from player import *
from random import *
import copy
from board import *
from minmax import *
from scoremap import *

class Node:
	def __init__(self, board, parent, move, player):
		self.board = board
		self.parent = parent
		self.value = 1
		self.childs = []
		self.move = move
		self.player = player

	def __repr__(self):
		if self.childs > 1:
			number=0
			for child in self.childs:
				number=number+1
			return "Has %s child(s)" % number
		return "STATE"

# This class tries to add some strategic value to the min max algorithme
class StrategicMinMax(MinMax):
	def __init__(self, screen, images, settings, difficulty):
		self.scoremap = ScoreMap()
		#self.scoremap.randomizeMap()
		MinMax.__init__(self, screen, images, settings, difficulty)

	def score(self, node, player, opponent):
		current_score    = 0

		scoremap = ScoreMap()

		for y in range( len(node.board.state[3]) ):
			if node.board.state[3][y] == opponent and y != 0:
				for x in range(7):
					scoremap.setStoneScore(x,y, scoremap.getStoneScore(x, y)+8)

			if node.board.state[3][0] == opponent:
				if ((len(node.board.state[2]) and node.board.state[2][0]==player) and (len(node.board.state[4]) and node.board.state[4][0]==opponent)):
					scoremap.setStoneScore(2,0, 80)
				if ((len(node.board.state[4]) and node.board.state[4][0]==player) and (len(node.board.state[2]) and node.board.state[2][0]==opponent)):
					scoremap.setStoneScore(4,0, 80)

		number_of_stones = 0

		for x in range(7):
			for y in range(len(node.board.state[x])):
				if node.board.state[x][y] == player:
					current_score += scoremap.getStoneScore(x, y)
					number_of_stones += 1
				"""
				else:
					current_score -= scoremap.getStoneScore(x, y)
					number_of_stones += 1
				"""
		return (current_score * 100) / number_of_stones


###############################################################################################
# $Id: main_dialog.py,v 1.1.1.1 2002/02/19 10:16:08 slmm Exp $
###############################################################################################

import pygame
import pygame.font
from pygame.locals import *
import scenario
import properties
from organization import REBEL, UNION, toString
import sys
import widget
from dialog          import *
from new_game        import NewGame
from button          import Button
from label           import Label
from info_scenario   import InfoScenario
from widget_manager  import WidgetManager
from start_game      import StartGame

class MainDialog(Dialog):
    """
    This class is used as the main dialog for the application. It is launched as the first graphical
    part of civil as soon as it starts. It presents the user with a basic dialog where some actions
    can be performed through clicking buttons:

    o start a new game with a new scenario
    o load a saved game (not yet implemented)
    o set options for the game
    o start the game
    o quit
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )


    def createWidgets (self):
        "Creates all widgets for the dialog."
        # labels
        self.wm.register ( Label (self.titlefont, "Welcome to Civil", (10,  10), color=(255, 0, 0)))
        self.wm.register ( Label (self.smallfont, "Scenario:",        (50, 180), color=(255, 255, 255)))
        self.wm.register ( Label (self.smallfont, "Play as: ",        (50, 210), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "Server name: ",    (50, 240), color=(255, 255, 255)) )
        self.wm.register ( Label (self.smallfont, "My name: ",        (50, 270), color=(255, 255, 255)))

        # create the changeable labels
        self.scenariolabel = Label (self.smallfont, "no scenario selected", (200, 180), color=(255, 255, 0))
        self.sidelabel     = Label (self.smallfont, "not yet set", (200, 210), color=(255, 255, 0))
        self.serverlabel   = Label (self.smallfont, scenario.connection.getHost (), (200, 240),
                                    color=(255, 255, 0))
        self.usernamelabel = Label (self.smallfont, scenario.local_player_name, (200, 270),
                                    color=(255, 255, 0))

        # register the labels for management
        self.wm.register ( self.serverlabel )
        self.wm.register ( self.scenariolabel )
        self.wm.register ( self.sidelabel )
        self.wm.register ( self.usernamelabel )

        # buttons
        self.wm.register ( Button ( properties.path_dialogs + "butt-new-game-moff.png",
                                    properties.path_dialogs + "butt-new-game-mover.png",
                                    (40, 650 ), {widget.MOUSEBUTTONUP : self.newGame } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-loadgame-moff.png",
                                    properties.path_dialogs + "butt-loadgame-mover.png",
                                    (282, 650 ), {widget.MOUSEBUTTONUP : self.loadGame } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-quit-moff.png",
                                    properties.path_dialogs + "butt-quit-mover.png",
                                    (772, 650 ), {widget.MOUSEBUTTONUP : self.quit } ) )


    def newGame (self, trigger, event):
        """Callback triggered when the user clicks the 'New game' button. """
        # create a dialog and run it
        dialog = NewGame ()
        state = dialog.run ()

        # was it accepted or rejected?
        if state == ACCEPTED:
            # dialog was ok:ed, we should now activate the button that allows us to start the game
            self.wm.register ( Button ( properties.path_dialogs + "butt-startgame-moff.png",
                                        properties.path_dialogs + "butt-startgame-mover.png",
                                        (528, 650 ), {widget.MOUSEBUTTONUP : self.startGame } ) )

            # do we noe have some info about the scenario?
            if scenario.info:
                # yep, update the label, create the text for the label

                # get the name and location
                name     = scenario.info.getName ()
                location = scenario.info.getLocation ()

                # get the scenario data
                self.lines = dialog.getChosenScenario ()

                # get the date and create a string
                (year, month, day, hour, minute) = scenario.info.getDate ()
                date = `year` + "." + `month` + "." + `day` + " " + `hour` + ":" + `minute`

                # create a label of the data and set it
                self.scenariolabel.setText ( name + ", " + location + ", " + date )

                # yep, so we need to update our display too, first set the labels
                self.sidelabel.setText ( toString ( scenario.local_player_id ) )
        
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


    def loadGame (self, trigger, event):
        """Callback triggered when the user clicks the 'Load game' button. """
        print "loadGame: not yet implemented"

     
    def quit (self, trigger, event):
        """Callback triggered when the user clicks the 'Quit' button. Will send the command 'quit'
        to the server and the quit. """
        # send our resignation
        scenario.connection.send ( 'quit\n' )

        # just go away
        pygame.quit ()
        sys.exit ( 0 )


    def startGame (self, trigger, event):
        """Callback triggered when the user clicks the 'Start game' button."""

        print "MainDialog.startGame: sending scenario."

        scenario.connection.send ( 'start %d %d %s\n' % ( (UNION, REBEL)[ scenario.local_player_id],
                                                          len ( self.lines ), scenario.local_player_name ) )
       
        for line in self.lines:
            scenario.connection.send ( line )

        print "MainDialog.startGame: sent scenario."
        

        # create the dialog
##         dialog = StartGame ()

##         # attempt to start the game
##         if dialog.run () != ACCEPTED:
##             # failed to start the game, get the error code
##             #error = dialog.getError ()
            
##             print "Failed to start the game" # + error[1]

##             # repaint the stuff in our dialog
##             self.wm.paint (force=1, clear=1)
##             return

##         print "MainDialog.startGame: all ok, game starting"
        
##         # repaint the stuff if needed
##         self.wm.paint (force=1, clear=1)

        # we're accepting the dialog the dialog
        self.state = ACCEPTED
        
        return widget.DONE
       

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
